<?php

/**
 * @file
 *
 * Defines the MaPS Log base class.
 */

namespace Drupal\maps\Log;

use Drupal\maps\Log\Context\Context;
use Drupal\maps\Log\Context\ContextInterface;
use Drupal\maps\Log\Observer\ObserverInterface;
use Drupal\maps\Exception\LogException;

/**
 * MaPS Log class.
 */
class Log implements LogInterface {

  /**
   * The DOM document.
   *
   * @var \DOMDocument
   */
  protected $domDocument;

  /**
   * The current context.
   *
   * @var ContextInterface
   */
  public $currentContext;

  /**
   * The log file name.
   *
   * @var string
   */
  protected $filename;

  /**
   * The log uri.
   *
   * @var string
   */
  protected $uri;

  /**
   * The full log path.
   *
   * @var string
   */
  protected $path;

  /**
   * The log type.
   *
   * @var string
   */
  protected $type;

  /**
   * Some contextual data related to the log.
   *
   * @var array
   */
  protected $relatedTokens = array();

  /**
   * The MaPS Log obervers.
   *
   * @var \SplObjectStorage
   */
  protected $observers;

  /**
   * The current operation (in case of batch processing).
   *
   * @var int
   */
  protected $currentOperation = 0;

  /**
   * The total number of operatiopns.
   *
   * @var int
   */
  protected $totalOperations = 1;

  /**
   * Class constructor
   *
   * @param string $type
   *   The type of log.
   * @param int id
   *   The log id.
   * @param boolean test
   *   Flag indicating if we are running tests.
   */
  public function __construct($type, $id = NULL, $test = FALSE) {
    $this->type = $type;

    if (is_null($id)) {
      $id = microtime();
    }

    $this->filename = hash('crc32', $type . $id) . '.xml';
    $this->uri = ($test === FALSE) ? file_default_scheme() : 'temporary';
    $this->uri .= '://' . $this->getLogDir() . '/' . $type ;

    $this->setPath($this->uri . '/' . $this->filename);
    $this->domDocument = new \DOMDocument();
    $this->observers = new \SplObjectStorage();

    if (is_file($this->getPath())) {
      $this->domDocument->load($this->getPath());
    }
    else {
      $this->initDomDocument();
    }
  }

  /**
   * Save the data to the XML file and dispatch the log to the observers.
   */
  public function save() {
    if ($this->writeXML()) {
      $this->dispatch();
    }
    else {
      watchdog('maps', 'MaPS Log: an error occured while writing the XML file. Context data: !data', array('!data' => (string) $this), WATCHDOG_ERROR);
    }
  }

  /**
   * Get the log directory.
   *
   * @return string
   */
  public function getLogDir() {
    return variable_get('maps_log_path', LogInterface::LOG_DIR);
  }

  /**
   * @inheritdoc
   */
  public function addContext(ContextInterface $context, $type = 'sibling') {
    $contextNode = $this->domDocument->createElement('context');

    // Get attributes
    $attributes = $context->getAttributes();
    if (!empty($attributes)) {
      foreach ($attributes as $name => $attribute) {
        $contextNode->setAttribute($name, $attribute);
      }
    }

    // Get current element
    if (is_null($this->currentContext)) {
      $elements = $this->domDocument->getElementsByTagName(self::CONTENT_ROOT);

      if (!$elements->length) {
        throw new LogException('Content node does not exist.', 0, array());
      }
      $current = $elements->item(0);
    }
    else {
      $current = $this->currentContext;
      if ($type === 'sibling') {
        $current = $current->parentNode;
      }
    }

    $current->appendChild($contextNode)
      ->setAttribute('level', WATCHDOG_DEBUG)
        ->parentNode
      ->setAttribute('type', $context->getType());

    $this->currentContext = $contextNode;

    return $this;
  }

  /**
   * @inheritdoc
   */
  public function addMessage($text, $level = WATCHDOG_DEBUG, array $variables = array(), $backtrace = FALSE, $links = array()) {
    if ($this->currentContext->tagName != 'context') {
      return FALSE;
    }

    if (is_array($text)) {
      $text = serialize($text);
    }

    // Text
    $messageNode = $this->domDocument->createElement('message');

    $messageNode->setAttribute('level', $level)
      ->parentNode
      ->appendChild($this->domDocument->createElement('text'))
        ->appendChild($this->domDocument->createCDATASection($text));

    // Variables
    if (!empty($variables)) {
      $messageNodeVariables = $this->domDocument->createElement('variables');

      foreach ($variables as $name => $variable) {
        $messageNodeVariables->appendChild($this->domDocument->createElement('variable'))
          ->setAttribute('name', $name)
            ->parentNode
          ->appendChild($this->domDocument->createCDATASection($variable));
      }
      $messageNode->appendChild($messageNodeVariables);
    }

    // Backtrace
    if ($backtrace) {
      $messageNode->appendChild($this->domDocument->createElement('backtrace', serialize(debug_backtrace())));
    }

    // Links
    if (!empty($links)) {
      $messageNodeLinks = $this->domDocument->createElement('links');

      foreach ($links as $link) {
        $messageNodeLinks->appendChild($this->domDocument->createElement('link'))
          ->appendChild($this->domDocument->createCDATASection($link));
      }
      $messageNode->appendChild($messageNodeLinks);
    }

    $this->currentContext->appendChild($messageNode);
    $this->updateContextsLevel($this->currentContext, $level);
    return $this;
  }

  /**
   * @inheritdoc
   */
  public function clearAndAddMessage($text, $level = WATCHDOG_DEBUG, array $variables = array(), $backtrace = FALSE, $links = array()) {
    // Delete messages.
    $this->deleteMessages();

    // Add new message.
    $this->addMessage($text, $level, $variables, $backtrace, $links);
    return $this;
  }

  /**
   * @inheritdoc
   */
  public function deleteMessages() {
    if (!empty($this->currentContext->childNodes)) {
      foreach ($this->currentContext->childNodes as $child) {
        $this->currentContext->removeChild($child);
      }
    }

    return $this;
  }

  /**
   * Delete the current context and set the internal pointer to this context parent
   *
   * @return Log
   */
  public function deleteContext() {
    $current = $this->currentContext;

    $this->moveToParent();
    $this->currentContext->removeChild($current);

    return $this;
  }

  /**
   * Move the internal pointer to the given child context.
   *
   * @param $contextName
   *    The name of the context we are looking for.
   *
   * @param $autoCreate
   *    If set to TRUE, if the context does not exists, it will be created as a child of the current context.
   *    Set to FALSE by default, because its uses are relatively specific.
   *
   * @return Log
   */
  public function goToContext($contextName, $autoCreate = FALSE) {
    if (!empty($this->currentContext->childNodes)) {
      foreach ($this->currentContext->childNodes as $child) {
        if ($child->getAttribute('type') == $contextName) {
          $this->currentContext = $child;

          return $this;
        }
      }
    }

    if ($autoCreate) {
      $this->addContext(new Context($contextName), 'child');
    }

    return $this;
  }

  /**
   * Add a message from a catched exception
   *
   * @param $e
   *    The catched exception.
   * @param $links
   *    An array of links.
   */
  public function addMessageFromException(\Exception $e, array $links = array()) {
    $variables = ($e instanceof Drupal\maps\Exception\MapsException) ? $e->getContext : array();
    $this->addMessage($e->getMessage(), $e->getCode(), $variables, TRUE, $links);

    return $this;
  }

  /**
   * Update the error level of a given context recursively:
   * Move to each parent and update the parent's level if necessary (if >=).
   *
   * @param $context
   *    The current context.
   * @param $level
   *    The error level.
   */
  public function updateContextsLevel($context, $level) {
    if ($context->getAttribute('level') >= $level) {
      $context->setAttribute('level', $level);

      if ($context->parentNode->tagName === 'context') {
        $this->updateContextsLevel($context->parentNode, $level);
      }
    }
  }

  /**
   * Set the internal pointer to the content root.
   *
   * @return Log
   */
  public function moveToContentRoot() {
    $this->currentContext = $this->domDocument->getElementsByTagName(self::CONTENT_ROOT)->item(0);
    return $this;
  }

  /**
   * @inheritdoc
   */
  public function moveToParent($contextType = NULL) {
    if (!is_null($contextType)) {
      if (!$this->currentContext->parentNode) {
        return FALSE;
      }

      if ($this->currentContext->getAttribute('type')) {
        do {
          $this->currentContext = $this->currentContext->parentNode;

          if ($this->currentContext->tagName === self::CONTENT_ROOT) {
            break;
          }
        } while ($contextType != $this->currentContext->getAttribute('type'));
      }
    }
    else {
      $this->currentContext = $this->currentContext->parentNode;
    }

    return $this;
  }

  /**
   * @inheritdoc
   */
  public function addObserver(ObserverInterface $observer) {
    $this->observers->attach($observer);
    return $this;
  }

  /**
   * Initialize the dom document with main necessary tags
   */
  public function initDomDocument() {
    global $base_url;

    $xslt = $this->domDocument->createProcessingInstruction('xml-stylesheet', 'type="text/xsl" href="' . url('admin/maps/log.xsl', array('absolute' => TRUE)) . '" version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"');

    $this->domDocument->appendChild($xslt)
      ->parentNode
      ->appendChild($this->domDocument->createElement('root'))
        ->appendChild($this->domDocument->createElement(self::CONTENT_ROOT));
  }

  /**
   * Magic method; cast to string.
   *
   * @return string
   *   The log URL.
   */
  public function __toString() {
    return file_create_url($this->getPath());
  }

  /**
   * Set the introduction message in the dom document
   *
   * @param $introductionMessage
   *  The content of the introduction message.
   *
   * @return LogInterface
   *   The current object.
   */
  public function setIntroductionMessage($introductionMessage) {
    $xpath = new \DOMXPath($this->domDocument);
    $root = $this->domDocument->getElementsByTagName('root')->item(0);
    $content = $this->domDocument->getElementsByTagName(self::CONTENT_ROOT)->item(0);

    $elements = $xpath->query('//root/introduction_message');
    if ($elements->length > 1) {
      throw new LogException('Multiple introduction message node: %introductionMessage', 0, array('%introductionMessage' => $introductionMessage));
    }
    else if ($elements->length == 1) {
      $elements->item(0)->parentNode->removeChild($elements->item(0));
    }

    // Introduction Message
    $introductionMessageNode = $this->domDocument->createElement('introduction_message');

    // The introduction message is supposed to be valid HTML. But...
    if (module_exists('filter')) {
      $introductionMessage = _filter_htmlcorrector($introductionMessage);
    }

    $introductionMessageXml = $this->domDocument->createDocumentFragment();
    $introductionMessageXml->appendXML('<html>' . $introductionMessage . '</html>');

    $introductionMessageNode->appendChild($introductionMessageXml);

    $content->parentNode->insertBefore($introductionMessageNode, $content);

    return $this;
  }

  /**
   * Get all messages for the current context
   *
   * @return $messages.
   *    An array of all messages for this context.
   */
  public function getMessages() {
    $messages = array();

    if (!empty($this->currentContext->childNodes)) {

      foreach ($this->currentContext->childNodes as $child) {
        if ($child->tagName === 'message') {
          $messages[] = $child->nodeValue;
        }
      }
    }

    return $messages;
  }

  /**
   * Set log path
   *
   * @param $path.
   *  The path to log files.
   */
  public function setPath($path) {
    $this->path = $path;
  }

  /**
   * Return filename
   *
   * @return string
   *  The file name.
   */
  public function getFilename() {
    return $this->filename;
  }

  /**
   * Return the log Path
   *
   * @return string
   *  The log path.
   */
  public function getPath() {
    return $this->path;
  }

  /**
   * Return the dom document
   *
   * @return DOMDocument
   */
  public function getDomDocument() {
    return $this->domDocument;
  }

  /**
   * Return the log type.
   *
   * @return string
   *    The log type.
   */
  public function getType() {
    return $this->type;
  }

  /**
   * Set the current operation count.
   *
   * @param int
   *    The number of the current operation
   */
  public function setCurrentOperation($currentOperation) {
    $this->currentOperation = (int) $currentOperation;
  }

  /**
   * Set the total opeartions count.
   *
   * @param int
   *    The number of total operations.
   */
  public function setTotalOperations($totalOperations) {
    $this->totalOperations = (int) $totalOperations;
  }


  /**
   * Return percentage of completion of the log related tasks.
   *
   * @param int
   *    The number of current operation.
   *
   * @param int
   *    The total count of operation.
   *
   * @return int
   */
  public function getPercentage() {
    return (int) $this->currentOperation / $this->totalOperations;
  }

  /**
   * Return introduction Message
   *
   * @return string
   */
  public function getIntroductionMessage() {
    $xpath = new \DOMXPath($this->domDocument);
    $elements = $xpath->query('//root/introduction_message');

    if (!$elements->length) {
      return '';
    }

    $introductionMessageNode = $elements->item(0);

    return $introductionMessageNode->nodeValue;
  }

  /**
   * Get the log contextual data.
   *
   * @inheritdoc
   */
  public function getRelatedTokens() {
    return $this->relatedTokens;
  }

  /**
   * Set a contextual data.
   *
   * @inheritdoc
   */
  public function setRelatedToken($key, $value) {
    $this->relatedTokens[$key] = $value;
    return $this;
  }

  /**
   * Dispatch the current log to the observers.
   */
  protected function dispatch() {
    $percentage = $this->getPercentage();
    $method = $percentage < 1 ? 'dispatchPartialLog' : 'dispatchLog';

    foreach ($this->observers as $observer) {
      $observer->{$method}($this);
    }
  }

  /**
   * Save the data to the XML file.
   *
   * @return $boolean
   *    Indicates if the writting has been successfull.
   */
  protected function writeXML() {
    if (!file_exists($this->uri)) {
      drupal_mkdir($this->uri, NULL, TRUE);
    }

    if ($path = drupal_realpath($this->getPath())) {
      return file_put_contents($path, $this->domDocument->saveXML());
    }

    return FALSE;
  }

  /**
   * Increment the current operation counter.
   */
  public function incrementCurrentOperation() {
    $this->currentOperation++;
  }

  /**
   * Return an accessible url.
   */
  public function getUrl() {
    return file_create_url($this->getPath());
  }

}
