<?php

/**
 * @file
 * Defines the MaPS Log interface.
 */

namespace Drupal\maps\Log;

use Drupal\maps\Log\Context\ContextInterface;
use Drupal\maps\Log\Observer\ObserverInterface;

/**
 * MaPS Log Interface
 */
interface LogInterface {

  /**
   * Default log directory.
   */
  const LOG_DIR = 'maps/log';

  /**
   * Tag name of the "content" tag, which contain all contexts
   */
  const CONTENT_ROOT = '__content__';

  /**
   * Add a new context.
   *
   * @param $definition
   *   The new context.
   * @param $type
   *
   * @return Drupal\maps\Log\LogInterface
   *   The current object.
   */
  public function addContext(ContextInterface $context, $type = 'sibling');

  /**
   * Add a log message.
   *
   * @param $text
   *   The text of the message. It may contain placeholders.
   * @param $level
   *   The error level.
   * @param $variables
   *   An array of variables to display for debugging purposes.
   * @param $backtrace
   *   Flag indicating whether a backtrace should be attached to the message or not.
   * @param $links
   *   An array of internal or external links, that may help to get further information
   *   about the message context.
   *
   * @return Drupal\maps\Log\LogInterface
   *   The current object.
   *
   * @see watchdog()
   */
  public function addMessage($text, $level = WATCHDOG_DEBUG, array $variables = array(), $backtrace = FALSE, $links = array());

  /**
   * Clear the existing messages and add a log message
   *
   * @see addContext
   */
  public function clearAndAddMessage($text, $level = WATCHDOG_DEBUG, array $variables = array(), $backtrace = FALSE, $links = array());

  /**
   * Delete all messages for the current context
   *
   * @return $this
   */
  public function deleteMessages();

  /**
   * Set the internal cursor to a parent context.
   *
   * @param $contexttype
   *   If specified, the cursor is set to the closest parent of the given context type.
   *
   * @return Drupal\maps\Log\LogInterface
   *   The current object.
   */
  public function moveToParent($contextType);

  /**
   * Add a Log Observer.
   *
   * @param $observer
   *   A Drupal\maps\Log\ObserverInterface object.
   *
   * @return Drupal\maps\Log\LogInterface
   *   The current object.
   */
  public function addObserver(ObserverInterface $observer);

  /**
   * Get the log contextual data.
   *
   * @return array
   */
  public function getRelatedTokens();

  /**
   * Set a contextual data.
   *
   * @param $key
   *   The data key.
   * @param $value
   *   The data.
   *
   * @return Drupal\maps\Log\LogInterface
   *   The current object.
   *
   * @todo Check if the token type exists.
   */
  public function setRelatedToken($key, $value);

}
