<?php

/**
 * @file
 * Administrative UI for converters management.
 */

use Drupal\maps_import\Converter\Converter;
use Drupal\maps_import\Converter\ConverterInterface;
use Drupal\maps_import\Operation\Entity\Delete\Delete as EntityDelete;
use Drupal\maps_import\Converter\Object as ConverterObject;
use Drupal\maps_import\Profile\Profile;

/**
 * Form builder; correspondence table form.
 *
 * @see maps_import_converters_overview_form_submit()
 * @ingroup forms
 */
function maps_import_converters_overview_form($form, &$form_state, Profile $profile, $type) {
  drupal_set_title(t('@type converter settings for the profile @profile', array('@type' => ucfirst($type), '@profile' => $profile->getTitle())));

  $form['converters'] = array('#tree' => TRUE);
  $form['#profile'] = $profile;
  $form['#converter_type'] = $type;

  foreach (maps_import_get_converters($profile->getPid()) as $cid => $converter) {
    // Filter by type.
    if (get_class($converter) !== 'Drupal\\maps_import\\Converter\\' . ucfirst($type)) {
      continue;
    }

    $form['converters'][$cid]['#converter'] = $converter;
    $form['converters'][$cid]['cid'] = array('#type' => 'value', '#value' => $converter->getCid());

    $form['converters'][$cid]['weight'] = array(
      '#type' => 'weight',
      '#delta' => 50,
      '#title_display' => 'invisible',
      '#title' => t('Converter weight'),
      '#default_value' => $converter->getWeight(),
    );

    $replace = array(
      '%maps_import_profile' => $profile->getPid(),
      '%maps_import_converter' => $converter->getCid(),
      '%type' => $type,
    );

    $form['converters'][$cid]['links'] = array(
      '#theme' => 'links',
      '#links' => array(
        'converter-edit' => array(
          'href' => strtr('admin/maps/profiles/%maps_import_profile/%type/%maps_import_converter/edit', $replace),
          'title' => t('edit'),
        ),
        'converter-delete' => array(
          'href' => strtr('admin/maps/profiles/%maps_import_profile/%type/%maps_import_converter/delete', $replace),
          'title' => t('delete'),
        ),
        'converter-filters' => array(
          'href' => strtr('admin/maps/profiles/%maps_import_profile/%type/%maps_import_converter/filters', $replace),
          'title' => t('filters'),
        ),
        'converter-mapping' => array(
          'href' => strtr('admin/maps/profiles/%maps_import_profile/%type/%maps_import_converter/mapping', $replace),
          'title' => t('mapping'),
        ),
      ),
    );
  }

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
  );

  return $form;
}

/**
 * Form submission handler; correspondence table form.
 *
 * @see maps_import_converters_overview_form()
 */
function maps_import_converters_overview_form_submit($form, &$form_state) {
  $values = $form_state['values']['converters'];
  $max_weight = 0;

  foreach ($values as $cid => $value) {
    $max_weight = max($max_weight, $value['weight']);

    if ($value['weight'] != $form['converters'][$cid]['#converter']->getWeight()) {
      drupal_write_record('maps_import_converter', $value, array('cid'));
    }
  }

  drupal_set_message(t('The changes have been saved.'));
}

/**
 * Returns HTML for the correspondence table form.
 *
 * @ingroup themeable
 */
function theme_maps_import_converters_overview_form($variables) {
  $form = $variables['form'];
  $rows = array();

  drupal_add_tabledrag('maps-import-converters-overview', 'order', 'sibling', 'converter-weight');

  foreach (element_children($form['converters']) as $cid) {
    $form['converters'][$cid]['weight']['#attributes']['class'] = array('converter-weight');

    $row = array();
    $row[] = $form['converters'][$cid]['#converter']->getTitle();
    $row[] = $form['converters'][$cid]['#converter']->getDescription();
    $row[] = $form['converters'][$cid]['#converter']->getEntityType();
    $row[] = $form['converters'][$cid]['#converter']->getBundle();
    $row[] = drupal_render($form['converters'][$cid]['weight']);
    $row[] = drupal_render($form['converters'][$cid]['links']);

    $rows[] = array(
      'data' => $row,
      'class' => array('draggable'),
    );
  }

  $output = theme('table', array(
    'header' => array(t('Title'), t('Description'),
      t('Entity type'), t('Bundle'), t('Weight'), t('Actions')),
    'rows' => $rows,
    'attributes' => array('id' => 'maps-import-converters-overview'),
    'empty' => t('There is no defined converter yet.'),
  ));

  return $output . drupal_render_children($form);
}

/**
 * Menu callback; add a new converter.
 *
 * @param Profile $profile
 *   The profile instance.
 * @param string $type
 *   The converter type.
 */
function maps_import_converter_add(Profile $profile, $type) {
  if ($converter = maps_import_converter_load('Drupal\\maps_import\\Converter\\' . maps_drupal2camelcase($type), $profile->getPid())) {
    drupal_set_title(maps_import_profile_title($profile, 'Add a converter for the profile @profile'));
    menu_set_active_item('admin/maps/profiles/' . $profile->getPid() . '/' . $type);
    $router_item = menu_get_item();
    $router_item['fake'] = TRUE;
    menu_set_item($_GET['q'], $router_item);
    return drupal_get_form('maps_import_converter_edit_form', $profile, $converter);
  }

  drupal_not_found();
  drupal_exit();
}

/**
 * Form builder; converter create/edit form.
 *
 * @see maps_import_converter_edit_form_validate()
 * @see maps_import_converter_edit_form_submit()
 * @ingroup forms
 */
function maps_import_converter_edit_form($form, &$form_state, Profile $profile, Converter $converter) {
  $form['pid'] = array('#type' => 'value', '#value' => $profile->getPid());
  $form['#converter'] = $converter;
  $options = $converter->getOptions();

  $form['title'] = array(
    '#type' => 'textfield',
    '#title' => t('Title'),
    '#description' => t('A natural name for this converter. You can always change this name later. Maximum number of characters allowed: @count.', array('@count' => 32)),
    '#default_value' => $converter->getTitle(),
    '#required' => TRUE,
    '#maxlength' => 32,
  );

  $form['cid'] = array(
    '#type' => 'hidden',
    '#default_value' => $converter->getCid(),
  );

  $form['description'] = array(
    '#type' => 'textfield',
    '#title' => t('Description'),
    '#default_value' => $converter->getDescription(),
    '#description' => t('A description of this converter.'),
    '#maxlength' => 255,
  );

  $form['class'] = array(
    '#type' => 'hidden',
    '#default_value' => get_class($converter),
  );

  $uids = array();
  foreach ($converter->getMapping()->getSourceProperties() as $key => $value) {
    $uids[$key] = $value->getTranslatedTitle();
  }

  $form['uid'] = array(
    '#title' => t('Unique identifier'),
    '#type' => 'select',
    '#options' => $uids,
    '#default_value' => $converter->getUid(),
    '#description' => t('Select an object property or attribute as unique identifier.'),
    '#required' => TRUE,
  );

  $form['uid_scope'] = array(
    '#title' => t('Scope for the unique identifier'),
    '#type' => 'select',
    '#options' => array(
      ConverterInterface::SCOPE_GLOBAL => t('Global'),
      ConverterInterface::SCOPE_PROFILE => t('Profile'),
    ),
    '#default_value' => $converter->getUidScope(),
    '#description' => t('Global scope means that objects from other converters or profiles may be overriden or replace fetched ones. Profile or converter scope means that only objects fetched through the current profile or converter may override each others.'),
    '#required' => TRUE,
  );

  // Edit converter.
  if (!is_null($converter->getEntityType())
      && !is_null($converter->getBundle())) {

    $entity_info = $converter->entityInfo();

    $form['_entity_type_'] = array(
      '#title' => t('Entity type'),
      '#value' => $entity_info[$converter->getEntityType()]['label'],
      '#disabled' => TRUE,
      '#type' => 'textfield',
    );

    $form['_bundle'] = array(
      '#title' => t('Bundle'),
      '#value' => $entity_info[$converter->getEntityType()]['bundles'][$converter->getBundle()]['label'],
      '#disabled' => TRUE,
      '#type' => 'textfield',
    );

    $form['entity_type'] = array(
      '#type' => 'hidden',
      '#value' => $converter->getEntityType(),
    );

    $form['bundle'] = array(
      '#type' => 'hidden',
      '#value' => $converter->getBundle(),
    );

    $form['bundles']['#tree'] = TRUE;
    $form['bundles'][$converter->getEntityType()] = array(
      '#type' => 'hidden',
      '#value' => $converter->getBundle(),
    );
  }
  else {
    // Add entity_type and bundle options.
    $form['entity_type'] = array(
      '#type' => 'select',
      '#title' => t('Entity type'),
      '#description' => t('Select the entity type.'),
      '#options' => array('' => '- None -'),
      '#required' => TRUE,
    );

    $form['bundle'] = array('#type' => 'value', '#value' => $converter->getBundle());
    $form['bundles']['#tree'] = TRUE;

    $presets = maps_import_image_presets($converter->getProfile());

    foreach ($converter->entityInfo() as $entity_type => $plugin) {
      $form['entity_type']['#options'][$entity_type] = $plugin['label'];

      // Remove image bundle if there is no preset defined.
      if ($entity_type == 1 && empty($presets)) {
        unset($plugin['bundles']['image']);
      }

      $form['bundles'][$entity_type] = array(
        '#type' => 'select',
        '#title' => t('Bundle type for entity %name', array('%name' => $plugin['label'])),
        '#description' => t('Select the bundle.'),
        '#options' => array('' => t('- Select -')) + maps_reduce_array($plugin['bundles'], 'label'),
        '#states' => array(
          'visible' => array(
            ':input[name="entity_type"]' => array('value' => $entity_type),
          ),
          'required' => array(
            ':input[name="entity_type"]' => array('value' => $entity_type),
          ),
        ),
      );

      if ($converter->getEntityType() === $entity_type) {
        $form['bundles'][$entity_type]['#default_value'] = $converter->getBundle();
      }
    }
  }

  // @todo add some options for UID, UID scope or
  // entity_type/bundle fields if their values
  // are changed on an existing converter.
  // What to do with the entities created by this
  // converter? Deleted?
  // Kept but deleting maps_import_entities related entries ?
  if ($converter->getType() === 'media' && !empty($presets)) {
    $form['preset'] = array(
      '#type' => 'select',
      '#title' => t('Preset'),
      '#default_value' => !empty($options['presets']) ? $options['presets'] : '',
      '#description' => t('Select the entity type.'),
      '#options' => array('' => '- Select -') + $presets,
      '#states' => array(
        'visible' => array(
          ':input[name="bundles[file]"]' => array('value' => 'image'),
        ),
        'required' => array(
          ':input[name="bundles[file]"]' => array('value' => 'image'),
        ),
      ),

    );
  }

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
  );

  return $form;
}

/**
 * Form validation handler; converter form.
 *
 * @see maps_import_converter_form()
 * @see maps_import_converter_form_submit()
 */
function maps_import_converter_edit_form_validate($form, &$form_state) {
  if ($entity_type = $form_state['values']['entity_type']) {
    if (empty($form_state['values']['bundles'][$entity_type])) {
      form_set_error('bundles][' . $entity_type, t('!name field is required.', array('!name' => $form['bundles'][$entity_type]['#title'])));
    }
    else {
      form_set_value($form['bundle'], $form_state['values']['bundles'][$entity_type], $form_state);
    }
  }
}

/**
 * Form submission handler; converter edit form.
 *
 * @see maps_import_converter_edit_form()
 * @see maps_import_converter_edit_form_validate()
 */
function maps_import_converter_edit_form_submit($form, &$form_state) {
  $form_state['redirect'] = array('admin/maps/profiles/' . $form_state['values']['pid'] . '/' . $form['#converter']->getType() . '/');

  if ((int) $form['#converter']->getCid() > 0) {
    $update = array('cid');
    $save_state = SAVED_UPDATED;
  }
  else {
    $update = array();
    $save_state = SAVED_NEW;
  }

  // Add some default values.
  $form_state['values'] += array(
    'options' => (!empty($form_state['values']['preset'])) ? array('presets' => $form_state['values']['preset']) : array(),
    'filters' => array(),
    'mapping' => array(),
  );

  if ($save_state !== drupal_write_record('maps_import_converter', $form_state['values'], $update)) {
    watchdog('maps', 'An error occured while creating/updating a MaPS Import converter.<hr />Values: !values<hr/>Converter: !converter.', array('!values' => maps_highlight_php($form_state['values']), '!converter' => maps_highlight_php($form['#converter'])), WATCHDOG_ERROR);
    drupal_set_message(t('An error occured while saving the converter.'), 'error');
    $form_state['redirect'] = array('admin/maps/profiles/' . $form_state['values']['pid'] . '/converters');
  }
  else {
    if ($save_state == SAVED_UPDATED) {
      // Get children converters if any.
      $list = db_select('maps_import_converter')
        ->fields('maps_import_converter', array('cid'))
        ->condition('parent_id', $form_state['values']['cid'])
        ->execute()
        ->fetchAllAssoc('cid');

      if (empty($list)) {
        return;
      }

      db_update('maps_import_converter')
        ->fields(array(
          'title' => $form_state['values']['title'],
        ))
        ->condition('cid', array_keys($list))
        ->execute();
    }
  }

}


/**
 * Form builder; converter options form.
 *
 * @see maps_import_converter_options_form_validate()
 * @see maps_import_converter_options_form_submit()
 * @ingroup forms
 */
function maps_import_converter_options_form($form, &$form_state, Profile $profile, Converter $converter) {
  drupal_set_title(t('Options for converter @converter', array('@converter' => $converter->getTitle())));

  $form_state['converter'] = $converter;
  $form['options'] = array('#tree' => TRUE) + $converter->getMapping()->optionsForm($form, $form_state);
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
  );

  return $form;
}

/**
 * Form validation handler; converter options form.
 *
 * @see maps_import_converter_options_form()
 * @see maps_import_converter_options_form_submit()
 */
function maps_import_converter_options_form_validate($form, &$form_state) {
  $form_state['converter']->getMapping()->optionsFormValidate($form, $form_state);
}

/**
 * Form submission handler; converter options form.
 *
 * @see maps_import_converter_options_form()
 * @see maps_import_converter_options_form_validate()
 */
function maps_import_converter_options_form_submit($form, &$form_state) {
  $form_state['converter']->getMapping()->optionsFormSubmit($form, $form_state);

  db_update('maps_import_converter')
    ->fields(array('options' => serialize($form_state['values']['options'])))
    ->condition('pid', $form_state['converter']->getProfile()->getPid())
    ->condition('cid', $form_state['converter']->getCid())
    ->execute();

  drupal_set_message(t('The changes have been saved.'));
}

/**
 * Form builder; converter deletion confirmation form.
 *
 * @see maps_import_converter_delete_confirm_form_submit()
 * @ingroup forms
 */
function maps_import_converter_delete_confirm_form($form, &$form_state, Profile $profile, Converter $converter) {
  drupal_set_title(t('Delete converter @converter', array('@converter' => $converter->getTitle())));

  $form['pid'] = array('#type' => 'value', '#value' => $profile->getPid());
  $form['cid'] = array('#type' => 'value', '#value' => $converter->getCid());
  $form['#converter'] = $converter;

  $options = array(
    0 => t('Delete converter and related filters and mapping items'),
    1 => t('Delete converter, related filters, mapping items and all entities created by this converter'),
  );

  $form['delete_entities'] = array(
    '#type' => 'radios',
    '#options' => $options,
    '#default_value' => 0,
  );

  return confirm_form(
    $form,
    t('Are you sure you want to delete the converter %title ?', array('%title' => $converter->getTitle())),
    'admin/maps/profiles/' . $profile->getPid() . '/' . $converter->getType(),
    '',
    t('Delete'),
    t('Cancel'));
}

/**
 * Form submission handler; delete the converter.
 *
 * @see maps_import_converter_delete_confirm_form()
 */
function maps_import_converter_delete_confirm_form_submit($form, &$form_state) {
  // Get children converters if any.
  $list = db_select('maps_import_converter')
    ->fields('maps_import_converter', array('cid'))
    ->condition('parent_id', $form_state['values']['cid'])
    ->execute()
    ->fetchAllAssoc('cid');
  $converters = array($form_state['values']['cid']);

  if (!empty($list)) {
    $converters = array_merge($converters, array_keys($list));
  }

  if ((int) $form_state['values']['delete_entities'] == 1) {
    $operation = new EntityDelete($form['#converter']);
    $batch = array(
      'operations' => $operation->batchOperations(),
      'title' => t('Deleting all entities created from converter %converter', array('%converter' => $form['#converter']->getTitle())),
    );

    $test_info = $GLOBALS['drupal_test_info'];
    if (!empty($test_info['test_run_id'])) {
      $operation->process();
    }
    else {
      batch_set($batch);
    }
  }

  $tables = array(
    'maps_import_converter',
    'maps_import_converter_conditions',
    'maps_import_mapping_item',
    'maps_import_entities',
  );

  foreach ($tables as $table) {
    db_delete($table)
      ->condition('cid', $converters)
      ->execute();
  }

  $form_state['redirect'] = array('admin/maps/profiles/' . $form_state['values']['pid'] . '/' . $form['#converter']->getType());
  drupal_set_message(t('The converter was successfully deleted!'));
}

/**
 * Form builder; media types correspondance table.
 *
 * @see maps_import_converter_media_settings_form_validate()
 * @see maps_import_converter_media_settings_form_submit()
 */
function maps_import_converter_media_settings_form($form, &$form_state, Profile $profile) {
  drupal_set_title(t('Media settings for profile @profile', array('@profile' => $profile->getTitle())));

  $media_types = maps_import_get_maps_media_types($profile);
  $form['#tree'] = TRUE;
  $form['#profile'] = $profile;

  $options = $profile->getOptions();

  foreach ($media_types as $key => $title) {
    $form['media_type'][$key]['title'] = array('#type' => 'item', '#markup' => $title);

    $form['media_type'][$key]['path'] = array(
      '#type' => 'textfield',
      '#default_value' => isset($options['media_settings'][$key]['path']) ? $options['media_settings'][$key]['path'] : '',
    );

    $form['media_type'][$key]['preset'] = array(
      '#type' => 'textfield',
      '#default_value' => isset($options['media_settings'][$key]['preset']) ? $options['media_settings'][$key]['preset'] : '',
    );

    if ($key > 1) {
      $form['media_type'][$key]['preset']['#attributes'] = array(
        'readonly' => 'readonly',
        'style' => 'background: none repeat scroll 0 0 #EAEAEA;',
      );
    }

  }

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
  );

  return $form;
}

/**
 * Form validation handler; sanitize the values.
 *
 * @see maps_import_converter_media_settings_form_validate()
 * @see maps_import_converter_media_settings_form_submit()
 */
function maps_import_converter_media_settings_form_validate($form, &$form_state) {
  foreach ($form_state['values']['media_type'] as $key => $media_type) {
    form_set_value($form['media_type'][$key]['path'], trim($media_type['path'], '\\/'), $form_state);
  }
}

/**
 * Form submission handler; save the settings.
 *
 * @see maps_import_converter_media_settings_form_validate()
 * @see maps_import_converter_media_settings_form_submit()
 */
function maps_import_converter_media_settings_form_submit($form, &$form_state) {
  $media_types = array();
  $values = $form_state['values']['media_type'];

  foreach ($values as $key => $value) {
    // Create the folder.
    $dir = "public://{$form['#profile']->getMediaDirectory()}/{$value['path']}";
    if (!is_dir($dir)) {
      mkdir($dir);
    }

    $media_types[$key]['path'] = $value['path'];
    $media_types[$key]['preset'] = $value['preset'];
  }

  $save = array(
    'pid' => $form['#profile']->getPid(),
    'options' => array('media_settings' => $media_types),
  );

  drupal_write_record('maps_import_profile', $save, array('pid'));
}

/**
 * Returns HTML for the media settings form.
 *
 * @ingroup themeable
 */
function theme_maps_import_converter_media_settings_form($variables) {
  $form = $variables['form'];
  $rows = array();

  if (!empty($form['media_type'])) {
    foreach (element_children($form['media_type'], TRUE) as $key) {
      $element = &$form['media_type'][$key];

      $row = array();
      $row[] = drupal_render($element['title']);
      $row[] = drupal_render($element['path']);
      $row[] = drupal_render($element['preset']);
      $rows[] = array(
        'data' => $row,
      );
    }
  }

  $output = theme('table', array(
    'header' => array(t('MaPS media type'), t('Directory path'), t('Preset')),
    'rows' => $rows,
    'empty' => t('There is no media types defined.'),
  ));

  return $output . drupal_render_children($form);
}
