<?php

/**
 * @file
 * Administrative functions for MaPS Import module
 */

use Drupal\maps_import\Profile\Profile;
use Drupal\maps_import\Request\Request;
use Drupal\maps_import\Fetcher\Fetcher;
use Drupal\maps_import\Cache\Object\Profile as CacheObjectProfile;

/**
 * Menu callback; profiles overview.
 */
function maps_import_admin_profiles() {
  $rows = array();

  foreach (CacheObjectProfile::getInstance()->loadAll() as $pid => $profile) {
    $row = array();
    $row[] = check_plain($profile->getTitle());
    $row[] = check_plain($profile->getPublicationId());
    $row[] = check_plain($profile->getRootObjectId());
    $row[] = drupal_strtoupper(check_plain($profile->getFormat()));
    $row[] = $profile->isEnabled() ? t('Enabled') : t('Disabled');
    $row[] = check_plain($profile->getMaxObjectsPerRequest());
    $row[] = check_plain($profile->getMaxObjectsPerOp());

    $actions = module_invoke_all('maps_import_profile_actions', $profile);
    uasort($actions, 'drupal_sort_weight');
    $row[] = theme('links', array('links' => $actions));

    $rows[] = $row;
  }

  if (!$rows) {
    $rows[] = array(
      array(
        'data' => t('There is no existing profile yet. Add a !link now.', array('!link' => l(t('new profile'), 'admin/maps/profiles/add'))),
        'colspan' => 6,
      ),
    );
  }

  $header = array(
    t('Title'),
    t('Publication'),
    t('Root object'),
    t('Format'),
    t('Status'),
    array(
      'data' => array(
        '#theme' => 'html_tag',
        '#tag' => 'dfn',
        '#value' => t('M.O./R.'),
        '#attributes' => array(
          'title' => t('Max objects per request'),
        ),
      ),
    ),
    array(
      'data' => array(
        '#theme' => 'html_tag',
        '#tag' => 'dfn',
        '#value' => t('M.O./O.'),
        '#attributes' => array(
          'title' => t('Max objects per mapping operation'),
        ),
      ),
    ),
    t('Actions'),
  );

  return theme('table', array('rows' => $rows, 'header' => $header));
}

/**
 * Form builder; MaPS Import profile add/edit form.
 *
 * @see maps_import_admin_profile_form_validate()
 * @see maps_import_admin_profile_form_submit()
 * @ingroup forms
 */
function maps_import_admin_profile_form($form, &$form_state, $profile = NULL) {
  $form['#profile'] = $profile;
  $schema = drupal_get_schema('maps_import_profile');
  $default = array();

  foreach ($schema['fields'] as $column => $info) {
    $default[$column] = isset($info['default']) ? $info['default'] : '';
  }

  if ($profile) {
    $default = array_merge($default, $profile->toArray());
    $form['pid'] = array('#type' => 'value', '#value' => $profile->getPid());
  }

  $form['title'] = array(
    '#type' => 'textfield',
    '#title' => t('Title'),
    '#default_value' => $default['title'],
    '#description' => t("The profile's title."),
    '#required' => TRUE,
  );

  $form['token'] = array(
    '#type' => 'textfield',
    '#title' => t('Security token'),
    '#default_value' => $default['token'],
    '#description' => t('This token is a 40-character secure hash used for authentication against MaPS System® Web services.'),
    '#maxlength' => 40,
    '#required' => TRUE,
  );

  $form['publication_id'] = array(
    '#type' => 'textfield',
    '#title' => t('Publication ID'),
    '#default_value' => $default['publication_id'],
    '#description' => t('Uses MaPS System® ID. Import process targets only object from the specified publication.'),
    '#element_validate' => array('maps_integer_element_validate'),
    '#not_null' => TRUE,
    '#positive' => 1,
    '#required' => TRUE,
  );

  $form['root_object_id'] = array(
    '#type' => 'textfield',
    '#title' => t('Root object ID'),
    '#default_value' => $default['root_object_id'],
    '#description' => t('Uses MaPS System® ID. Related root object is not imported, but used as a starting point for import process.'),
    '#element_validate' => array('maps_integer_element_validate'),
    '#not_null' => TRUE,
    '#positive' => 1,
    '#required' => TRUE,
  );

  $form['preset_group_id'] = array(
    '#type' => 'textfield',
    '#title' => t('Preset group ID'),
    '#default_value' => $default['preset_group_id'],
    '#description' => t('Uses MaPS System® ID. This settings defines the group of image presets which is used for Web publication.'),
    '#element_validate' => array('maps_integer_element_validate'),
    '#not_null' => TRUE,
    '#positive' => 1,
    '#required' => TRUE,
  );

  $form['url'] = array(
    '#type' => 'textfield',
    '#title' => t('MaPS System® Web Services URL'),
    '#default_value' => $default['url'],
    '#required' => TRUE,
  );

  $form['media'] = array(
    '#type' => 'fieldset',
    '#title' => t('Media configuration'),
  );

  $form['media']['media_directory'] = array(
    '#type' => 'textfield',
    '#title' => t('Media directory'),
    '#default_value' => $default['media_directory'],
    '#description' => t('The relative path inside the Drupal public folder.'),
  );

  $form['limits'] = array(
    '#type' => 'fieldset',
    '#title' => t('Objects number limits'),
    '#description' => t('The value %value should only be used for debug purposes and can only be selected if the site is in maintenance mode.', array('%value' => 1)),
  );

  $form['limits']['max_objects_per_request'] = array(
    '#type' => 'select',
    '#title' => t('Maximum number of objects per request'),
    '#default_value' => $default['max_objects_per_request'],
    '#options' => Profile::getMaxObjectOptions('request'),
    '#description' => t('This defines the maximum number of objects that may be returned by each HTTP request to MaPS System® Web Services.'),
    '#required' => TRUE,
  );

  $form['limits']['max_objects_per_op'] = array(
    '#type' => 'select',
    '#title' => t('Maximum number of objects to process per mapping operation'),
    '#default_value' => $default['max_objects_per_op'],
    '#options' => Profile::getMaxObjectOptions('mapping'),
    '#description' => t('This defines the maximum number of objects that may be processed by each mapping operation.'),
    '#required' => TRUE,
  );

  $form['format'] = array(
    '#type' => 'select',
    '#title' => t('Response format'),
    '#options' => array(
      Request::FORMAT_XML => t('XML'),
    ),
    '#default_value' => array($default['format']),
  );

  $form['enabled'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Status'),
    '#options' => array(1 => t('Enabled')),
    '#default_value' => array($default['enabled']),
    '#description' => t('Inactive profiles are skipped from import process.'),
  );

  $form['web_template'] = array(
    '#type' => 'select',
    '#title' => t('Web Template'),
    '#options' => array(
      Fetcher::TEMPLATE_EXPORT_WEB => t('Template export Web'),
    ),
    '#description' => 'The name of the web template to use in the MaPS System Webervices parameters.',
  );

  $form['actions'] = array('#type' => 'actions');

  $form['actions']['submit'] = array(
    '#type' => 'submit',
    '#value' => $profile ? t('Update') : t('Save'),
  );

  if ($profile) {
    $form['actions']['delete'] = array(
      '#markup' => l(t('delete'), 'admin/maps/profiles/' . $profile->getPid() . '/delete'),
      '#weight' => 10,
    );
  }

  return $form;
}

/**
 * Form validation handler; MaPS Import profile add/edit form.
 *
 * @see maps_import_admin_profile_form()
 * @see maps_import_admin_profile_form_submit()
 */
function maps_import_admin_profile_form_validate($form, &$form_state) {
  if (!valid_url($form_state['values']['url'])) {
    form_set_error('url', t('Field %field: please specify a valid URL.', array('%field' => $form['url']['#title'])));
  }

  // Title should be unique.
  if ($form_state['values']['title'] !== $form['title']['#default_value']) {
    $query = db_select('maps_import_profile', 'mws')
      ->fields('mws', array('pid'))
      ->condition('mws.title', $form_state['values']['title']);
    if ($query->execute()->fetchCol()) {
      form_set_error('title', t('The specified title is already in use.'));
    }
  }

  // Validates secure token.
  if (isset($form_state['values']['token']) && strlen($form_state['values']['token'])) {
    // Force lowercase.
    $token = drupal_strtolower($form_state['values']['token']);

    // Secure token should be exactly 40-character
    // long and use hexadecimal characters only.
    if (!preg_match('/^[0-9a-f]{40}$/', $token)) {
      form_set_error('token', t('The authentication token should be exactly 40-character long and only contain hexadecimal characters.'));
    }
    // Corrects case if necessary.
    elseif (strcmp($token, $form_state['values']['token'])) {
      form_set_value($form['token'], $token, $form_state);
    }
  }

  // Validate media directory.
  if (!empty($form_state['values']['media_directory'])) {
    $media_directory = trim($form_state['values']['media_directory'], '/');

    if (!strlen($media_directory)) {
      form_set_error('media_directory', t('The media directory cannot be the same as Drupal file directory.'));
    }
    elseif (!preg_match('/^^[a-z0-9\/]+$/i', $media_directory)) {
      form_set_error('media_directory', t('The media directory %value is not valid. It should only contains letters, digits and forward slashes.', array('%value' => $media_directory)));
    }
    else {
      $dir = variable_get('file_public_path', conf_path() . '/files') . '/' . $media_directory;

      // Use file_exists() to take care of syslinks.
      if (!file_exists($dir)) {
        form_set_error('media_directory', t('The media directory @dir does not exist.', array('@dir' => $dir)));
      }
    }
  }

  // Validates limits.
  $maintenance_mode = variable_get('maintenance_mode', 0);

  foreach (element_children($form['limits']) as $key) {
    if (!empty($form_state['values'][$key]) && $form_state['values'][$key] == 1 && !$maintenance_mode) {
      form_set_error($key, t('The value %value should only be used for debug purposes and can only be selected if the site is in maintenance mode.', array('%value' => 1)));
    }
  }
}

/**
 * Form submission handler; MaPS Import profile add/edit form.
 *
 * @see maps_import_admin_profile_form()
 * @see maps_import_admin_profile_form_validate()
 */
function maps_import_admin_profile_form_submit($form, &$form_state) {
  $pks = array();
  if (!empty($form_state['values']['pid'])) {
    $pks[] = 'pid';
  }

  // Flatten status value.
  $form_state['values']['enabled'] = reset($form_state['values']['enabled']);

  if (empty($form_state['values']['options'])) {
    $form_state['values']['options'] = array();
  }

  drupal_write_record('maps_import_profile', $form_state['values'], $pks);

  // Clear the cache.
  CacheObjectProfile::getInstance()->clearBinCache();

  $form_state['redirect'] = array('admin/maps/profiles');

  $t_args = array('@title' => $form_state['values']['title']);
  $message = $pks ? t('The profile @title has been updated.', $t_args) : t('The profile @title has been created.', $t_args);
  drupal_set_message(check_plain($message));
}

/**
 * Form builder; comfirmation form for profile deletion.
 *
 * @see maps_import_admin_profile_delete_confirm_form_submit()
 * @ingroup forms
 */
function maps_import_admin_profile_delete_confirm_form($form, &$form_state, Profile $profile) {
  $form['#profile'] = $profile;
  return confirm_form($form,
    t('Are you sure you want to delete the profile @title?', array('@title' => $profile->getTitle())),
    'admin/maps/profiles',
    t('This action cannot be undone.'),
    t('Delete'));
}

/**
 * Form submission handler; comfirmation form for profile deletion.
 *
 * @see maps_import_admin_profile_delete_confirm_form()
 */
function maps_import_admin_profile_delete_confirm_form_submit($form, &$form_state) {
  db_delete('maps_import_profile')
    ->condition('pid', $form['#profile']->getPid())
    ->execute();

  drupal_set_message(t('The profile %title has been deleted.', array('%title' => $form['#profile']->getTitle())));
  $form_state['redirect'] = array('admin/maps/profiles');

  // Clear the cache.
  CacheObjectProfile::getInstance()->clearBinCache();
}
