<?php

/**
 * @file
 * Abstract class that represents a converter.
 */

namespace Drupal\maps_import\Converter;

use Drupal\maps_import\Exception\ConverterException;
use Drupal\maps_import\Filter\FilterInterface;
use Drupal\maps_import\Filter as Filter;
use Drupal\maps_import\Profile\Profile;

abstract class Converter implements ConverterInterface {

  /**
   * The converter ID.
   *
   * @var int
   */
  private $cid = NULL;

  /**
   * The MaPS profile ID.
   *
   * @var Profile
   */
  private $profile;

  /**
   * The converter title.
   *
   * @var string
   */
  private $title = '';

  /**
   * The converter description.
   *
   * @var string
   */
  private $description = '';

  /**
   * The unique ID for imported objects.
   *
   * @var string
   */
  private $uid = NULL;

  /**
   * The unique ID scope.
   *
   * @var string
   */
  private $uid_scope = self::SCOPE_GLOBAL;

  /**
   * The entity type.
   *
   * @var string
   */
  private $entity_type = NULL;

  /**
   * The entity bundle.
   *
   * @var string
   */
  private $bundle = NULL;

  /**
   * The converter weight.
   *
   * @var int
   */
  private $weight = 0;

  /**
   * The converter options.
   *
   * @var array
   */
  private $options = array();

  /**
   * The converter filter
   *
   * @var FilterInterface
   */
  private $filter = NULL;

  /**
   * The parent id.
   *
   * @var int
   */
  private $parentId = 0;

  /**
   * @inheritdoc
   */
  public function __construct(Profile $profile) {
    $this->profile = $profile;
  }

  /**
   * @inheritdoc
   */
  public function getCondition($condition_id) {
    foreach ($this->getFilter()->getFlattenConditions() as $condition) {
      if ((int) $condition->getId() === (int) $condition_id) {
        return $condition;
      }
    }
  }

  /**
   * Get the conditions that are also containers.
   *
   * @return array
   *   An array whose keys are container ids and whose values are
   *   container titles.
   */
  public function getConditionContainers() {
    $containers = array();

    foreach ($this->getFilter()->getFlattenConditions() as $key => $condition) {
      if ($condition->isContainer()) {
        // @todo use getter and setter for "depth".
        $containers[$condition->getId()] = str_repeat('--', $condition->depth)  . $condition->getLabel();
      }
    }//end foreach

    return $containers;
  }

  /**
   * @inheritdoc
   */
  public function getCid() {
    return $this->cid;
  }

  /**
   * @inheritdoc
   */
  public function setCid($cid) {
    $this->cid = $cid;
  }

  /**
   * @inheritdoc
   */
  public function getPid() {
    return $this->getProfile()->getPid();
  }

  /**
   * @inheritdoc
   */
  public function getProfile() {
    return $this->profile;
  }

  /**
   * @inheritdoc
   */
  public function getTitle() {
    return $this->title;
  }

  /**
   * @inheritdoc
   */
  public function setTitle($title) {
    $this->title = $title;
  }

  /**
   * @inheritdoc
   */
  public function getDescription() {
    return $this->description;
  }

  /**
   * @inheritdoc
   */
  public function setDescription($description) {
    $this->description = $description;
  }

  /**
   * @inheritdoc
   */
  public function getUid() {
    return $this->uid;
  }

  /**
   * @inheritdoc
   */
  public function setUid($uid) {
    $this->uid = $uid;
  }

  /**
   * @inheritdoc
   */
  public function getUidScope() {
    return (int) $this->uid_scope;
  }

  /**
   * @inheritdoc
   */
  public function setUidScope($uid_scope) {
    $this->uid_scope = $uid_scope;
  }

  /**
   * @inheritdoc
   */
  public function getEntityType() {
    return $this->entity_type;
  }

  /**
   * @inheritdoc
   */
  public function setEntityType($entity_type) {
    $this->entity_type = $entity_type;
  }

  /**
   * @inheritdoc
   */
  public function getBundle() {
    return $this->bundle;
  }

  /**
   * @inheritdoc
   */
  public function setBundle($bundle) {
    $this->bundle = $bundle;
  }

  /**
   * @inheritdoc
   */
  public function getWeight() {
    return $this->weight;
  }

  /**
   * @inheritdoc
   */
  public function setWeight($weight) {
    $this->weight = $weight;
  }

  /**
   * @inheritdoc
   */
  public function getOptions() {
    return $this->options;
  }

  /**
   * @inheritdoc
   */
  public function setOptions($options) {
    $this->options = $options;
  }

  /**
   * @inheritdoc
   */
  public function getFilter() {
    $records = maps_get_records('maps_import_converter_conditions', 'id', array('cid' => (int) $this->getCid()), \PDO::FETCH_ASSOC);
    $data = array();

    foreach ($records as $element) {
      $data[] = new $element['class']($this, $element);
    }

    $filterClass = 'Drupal\\maps_import\\Filter\\' . maps_drupal2camelcase($this->getType());
    $this->filter = new  $filterClass($data);

    return $this->filter;
  }

  /**
   * Unset the Filter instance.
   */
  public function unsetFilter() {
    if (isset($this->filter)) {
      unset($this->filter);
    }
  }

  /**
   * Get information about entity that are supported by
   * Entity API.
   *
   * @return array
   */
  public function entityInfo() {
    return array_intersect_key(entity_get_info(), entity_get_property_info());
  }

  /**
   * Get some configuration items.
   *
   * @param $types
   *   Either an array of settings types or a string.
   *
   * @return array
   *   An array of settings grouped by type then keyed
   *   by their ID.
   */
  public function getConfigSettings($types) {
    static $settings = array();

    if (is_array($types)) {
      $types = array_unique(array_filter($types));
      sort($types);
      $key = implode(':', $types);
    }
    else {
      $key = $types;
    }

    if (!isset($settings[$key])) {
      $settings[$key] = $this->getProfile()->getConfigurationTypes($types, $this->getProfile()->getLanguage());
    }

    return $settings[$key];
  }


  /**
   * @inheritdoc
   */
  public function getParentId() {
    return 0;
  }

}
