<?php

/**
 * @file
 * Interface for MaPS Import Converter
 */

namespace Drupal\maps_import\Converter;

use Drupal\maps_import\Profile\Profile;
use Drupal\maps_import\Filter\FilterInterface;
use Drupal\maps_import\Mapping\Source\MapsSystem\Entity;

/**
 * The Converter interface.
 */
interface ConverterInterface {

  /**
   * Defines the uid scope for a child converter.
   */
  const SCOPE_INHERIT = 0;

  /**
   * Defines the global scope for unique IDs.
   */
  const SCOPE_GLOBAL = 1;

  /**
   * Defines the profile scope for unique IDs.
   */
  const SCOPE_PROFILE = 2;

  /**
   * Class constructor.
   *
   * @param $profile
   *   The MaPS Import Profile instance.
   */
  public function __construct(Profile $profile);

  /**
   * Get all mapping plugins.
   *
   * Each mapping plugin relates to a specific entity.
   */
  public function entityInfo();

  /**
   * Get the converter id.
   *
   * @return int
   *   The converter ID.
   */
  public function getCid();

  /**
   * Set the converter id.
   *
   * @param $cid
   *   The converter ID.
   */
  public function setCid($cid);

  /**
   * Get the related profile ID.
   *
   * @return int
   *   The profile id.
   */
  public function getPid();

  /**
   * Get the converter profile.
   *
   * @return Profile
   *   The converter profile.
   */
  public function getProfile();

  /**
   * Get the converter title.
   *
   * @return string
   *   The converter title.
   */
  public function getTitle();

  /**
   * Set the converter title.
   *
   * @param $title.
   *    The converter title.
   */
  public function setTitle($title);

  /**
   * Get the converter description.
   *
   * @return string
   *   The converter description.
   */
  public function getDescription();

  /**
   * Set the converter description.
   *
   * @param $description
   *    The converter description.
   */
  public function setDescription($description);

  /**
   * Get the converter uid.
   *
   * @return string
   *   The converter uid.
   */
  public function getUid();

  /**
   * Set the converter uid.
   *
   * @param $uid
   *    The converter uid.
   */
  public function setUid($uid);

  /**
   * Get the converter uid scope.
   *
   * @return int
   *   The converter uid scope.
   */
  public function getUidScope();

  /**
   * Set the converter uid scope.
   *
   * @param $uid_scope
   *    The converter uid scope.
   */
  public function setUidScope($uid_scope);

  /**
   * Get the converter entity type.
   *
   * @return string
   *   The converter entity type.
   */
  public function getEntityType();

  /**
   * Set the converter entity type.
   *
   * @param $entity_type.
   *    The converter entity type.
   */
  public function setEntityType($entity_type);

  /**
   * Get the converter bundle.
   *
   * @return string
   *   The converter bundle.
   */
  public function getBundle();

  /**
   * Set the converter bundle.
   *
   * @param $bundle
   *    The converter bundle.
   */
  public function setBundle($bundle);

  /**
   * Get the converter weight.
   *
   * @return int
   *   The converter weight.
   */
  public function getWeight();

  /**
   * Set the converter weight.
   *
   * @param $weight
   *    The converter weight.
   */
  public function setWeight($weight);

  /**
   * Get the converter options.
   *
   * @return array
   *   The converter options.
   */
  public function getOptions();

  /**
   * Set the converter options.
   *
   * @param $options
   *    The converter options.
   */
  public function setOptions($options);

  /**
   * Get a specific condition rom its ID.
   *
   * @param $condition_id
   *   The MaPS Import Filter Condition ID.
   *
   * @return Condition
   */
  public function getCondition($condition_id);

  /**
   * Return the related mapping object.
   *
   * @return MappingInterface
   */
  public function getMapping();

  /**
   * Return the converter type.
   *
   * @return string
   */
  public function getType();

  /**
   * Get the related Filter instance.
   *
   * @return  FilterInterface
   */
  public function getFilter();

  /**
   * Check if the mapping is allowed.
   *
   * @return boolean
   */
  public function isMappingAllowed();

  /**
   * Return the parent id.
   *
   * @return int
   */
  public function getParentId();

}
