<?php

/**
 * @file
 * Handles configuration import operation.
 */

namespace Drupal\maps_import\Fetcher;

use Drupal\maps_import\Profile\Profile;
use Drupal\maps_import\Exception\OperationException;
use Drupal\maps\Log\Logger;

/**
 * MaPS Import Configuration fetcher class.
 */
class Configuration extends Fetcher {

  /**
   * The configuration type related to an attribute-object.
   *
   * @var string
   */
  const ATTRIBUTE_OBJECT = 'attribute';

  /**
   * The configuration type related to a linked object.
   *
   * @var string
   */
  const LINKED_OBJECT = 'linked';

  /**
   * @inheritdoc
   */
  public function __construct(Profile $profile) {
    parent::__construct($profile);
    $this->setArguments(array(
      'config' => TRUE,
      'presetsGroup' => $profile->getPresetGroupId(),
    ));
  }

  /**
   * @inheritdoc
   */
  public function getTitle() {
    return t('Configuration import');
  }

  /**
   * @inheritdoc
   */
  public function getDescription() {
    return t('The configuration import operation is related to the profile %profile. It first performs a request against MaPS System® Web services to get data related to configration, like language list, image presets, library items etc. Then it parses the fetched data and finally stores them in the configuration tables of the database.', array('%profile' => $this->getProfile()->getTitle()));
  }

  /**
   * @inheritdoc
   */
  public function getType() {
    return 'configuration_fetch';
  }

  protected function fetch(&$context = NULL) {
    $this->setTimeout(self::TIMEOUT);
    return parent::fetch($context);
  }

  /**
   * Gets information about configuration elements.
   *
   * @return array
   *   Array whose keys are parsed data root element and whose values
   *   are the type of data.
   */
  protected function info() {
    return array(
      'languages'         => 'language',
      'units'             => 'unit',
      'link_types'        => 'link_type',
      'classes'           => 'class',
      'object_natures'    => 'object_nature',
      'object_types'      => 'object_type',
      'presets'           => 'image_preset',
      'attributes'        => 'attribute',
      'media_types'       => 'media_type',
      'libraries'         => 'library',
      'statuses'          => 'status',
      'object_attribute'  => 'attribute_object', // An object used as an attribute.
      'object_linked'     => 'attribute_object', // An linked object outside the publication scope.
    );
  }

  protected function save() {
    $info = $this->info();

    // Open a transaction to ensure data integrity.
    $transaction = db_transaction();

    try {
      // Remove old data for current profile.
      db_delete($this::DB_CONF_TABLE)->condition('pid', $this->getProfile()->getPid())->execute();
      db_delete($this::DB_LIBRARY_TABLE)->condition('pid', $this->getProfile()->getPid())->execute();

      foreach ($info as $key => $type) {
        if (!empty($this->data[$key])) {
          // Remember that function and method names are not case-sensitive, but
          // however we modify come characters case, only for design.
          $method = 'dataSave' . maps_drupal2camelcase($key);

          $callback = method_exists($this, $method) ? array($this, $method) : array($this, 'dataSave');
          call_user_func_array($callback, array($this->data[$key], $type));
        }
      }//end foreach

      // Clear object and media properties cache for current profile.
      cache_clear_all('object-properties:' . $this->getProfile()->getPid() . ':', 'cache_maps', TRUE);
      cache_clear_all('media-properties:' . $this->getProfile()->getPid() . ':', 'cache_maps', TRUE);
    }
    catch(Exception $e) {
      $transaction->rollback();

      if ($e instanceof OperationException) {
        $e->addContext('$profile', $this->getProfile());
        $e->watchdog();
      }
      else {
        watchdog_exception('maps', $e);
      }

      if ($this->isBatch()) {
        $this->context['results']['error'] = $e->getMessage();
        throw $e;
      }

      return FALSE;
    }

    return TRUE;
  }

  protected function processRun($index = 0, $range = 0, $last = FALSE) {
     if ($range > 0) {
      $this->setArguments(array(
        'startIndex' => $index,
        'maxResults' => $range,
      ));
      Logger::getLog($this->getType())->setCurrentOperation(ceil($index / $range) +1);
      Logger::getLog($this->getType())->setTotalOperations(ceil($this->getTotalOperations() / $range));
    }
    else {
      Logger::getLog($this->getType())->setCurrentOperation(1);
      Logger::getLog($this->getType())->setTotalOperations(1);
    }

  	parent::processRun($index, $range, $last);
    Logger::getLog($this->getType())->save();
  }

  /**
   * Saves "object_linked" data.
   */
  protected function dataSaveObjectLinked(array $data, $type) {
    $this->saveObjects($data, self::LINKED_OBJECT);
  }

  /**
   * Saves "object_attribute" data.
   *
   * Those objects are saved in the {maps_import_objects} and not in
   * a configuration table in order to be converted as standard objects.
   */
  protected function dataSaveObjectAttribute(array $data, $type) {
    $this->saveObjects($data, self::ATTRIBUTE_OBJECT);
  }

  /**
   * Saves "object_attribute" data.
   */
  protected function dataSaveLibraries(array $data, $type) {
    foreach ($data as $id_attribute => $library) {
      foreach ($library as $id => $item) {
        foreach ($item['values'] as $id_language => $values) {
        	$value = reset($values);
          db_insert($this::DB_LIBRARY_TABLE)
            ->fields(array(
              'pid' => $this->getProfile()->getPid(),
              'id' => $id,
              'code' => $item['code'],
              'id_attribute' => $id_attribute,
              'id_language' => $id_language,
              'value' => $value['value'],
            ))
            ->execute();

          $this->incrementCounter('insert', $type);
        }//end foreach
      }//end foreach
    }//end foreach
  }

  /**
   * Saves "statuses" data.
   */
  protected function dataSaveStatuses(array $data, $type) {
    foreach ($data as $id => $status) {
      db_insert($this::DB_CONF_TABLE)
        ->fields(array(
          'pid' => $this->getProfile()->getPid(),
          'type' => 'status',
          'id' => $id,
          'code' => $status['value'],
          'title' => $status['value'],
        ))
        ->execute();

      $this->incrementCounter('insert', $type);
    }//end foreach
  }

  /**
   * Saves "presets" data.
   */
  protected function dataSavePresets(array $data, $type) {
    foreach ($data as $id => $preset) {
      db_insert($this::DB_CONF_TABLE)
        ->fields(array(
          'pid' => $this->getProfile()->getPid(),
          'type' => $type,
          'id' => $id,
          'code' => $id,
          'title' => $preset['name']['value'],
        ))
        ->execute();

      $this->incrementCounter('insert', $type);
    }//end foreach
  }

  /**
   * Saves a piece of data into the database.
   *
   * This default method should only be used on standard formatted data,
   * whose structure is:
   * array(
   *   (int) id => array(
   *     (string) attribute1 => mixed value,
   *     ...
   *     'values' => array(
   *       (int or 'und') id_language => array(
   *         (string) attributeN => mixed value,
   *         ...
   *         'value' => (string) translated title,
   *       ),
   *     ),
   *   )
   * )
   */
  protected function dataSave(array $data, $type) {
    foreach ($data as $id => $datum) {
      $ignore = array_flip(array('code', 'values', 'value'));
      $fields = array(
        'pid' => $this->getProfile()->getPid(),
        'type' => $type,
        'id' => $id,
        'code' => isset($datum['code']) ? $datum['code'] : '',
      );

      foreach ($datum['values'] as $id_language => $values) {
      	$value = reset($values);
        $fields['id_language'] = $id_language;
        $fields['title'] = isset($value['value']) ? $value['value'] : '';
        $fields['data'] = serialize(array_diff_key(array_merge($datum, $value), $ignore));

        db_insert($this::DB_CONF_TABLE)
          ->fields($fields)
          ->execute();

        $this->incrementCounter('insert', $type);
      }//end foreach
    }//end foreach
  }

}
