<?php

/**
 * @file
 * Handle mapping operation.
 */

namespace Drupal\maps_import\Mapping\Mapper;

use Drupal\maps_import\Operation\Operation;
use Drupal\maps_import\Profile\Profile;
use Drupal\maps_import\Fetcher\Fetcher;
use Drupal\maps_import\Mapping\Source\MapsSystem\EntityInterface;
use Drupal\maps_import\Mapping\Source\MapsSystem\PropertyWrapper;
use Drupal\maps_import\Cache\Object\Converter as CacheObjectConverter;
use Drupal\maps\Log\Logger;
use Drupal\maps\Log\Context\Context as LogContext;

/**
 * Base class for all mapping operations.
 */
abstract class Mapper extends Operation {

	/**
	 * The current log.
	 */
	public static $log;

  /**
   * The list of converters.
   *
   * @var ArrayObject
   */
  protected $converters;

  /**
   * The current mapping type.
   *
   * @var string
   */
  protected static $currentMappingType;

  /**
   * Class constructor.
   *
   * @param $profile
   *   The profile instance.
   *
   * @return Mapper
   */
  public function __construct(Profile $profile) {
    $this->profile = $profile;
  }

  /**
   * Get the applicable converters.
   */
  protected function getConverters() {
    if (!isset($this->converters)) {
      $this->converters = new \ArrayObject();

      foreach (maps_import_get_converters($this->getProfile()->getPid()) as $converter) {
      	if (is_a($converter, $this->getConverterClass()) && !method_exists($converter, 'getParent')) {
        	$this->converters->append($converter);
        }
      }//end foreach
    }

    return $this->converters;
  }

  /**
   * Get the expected converter class.
   */
  protected abstract function getConverterClass();

  /**
   * Get the table name that stores the entity fetched from
   * MaPS System®.
   */
  protected abstract function getEntityTable();

  /**
   * Return the current mapping type;
   *
   * @return string
   */
  public static function getCurrentMappingType() {
    return self::$currentMappingType;
  }

  /**
   * Get the total number of objects to map.
   */
  public function getTotal() {
  	return db_query('SELECT COUNT(*) FROM {' . $this->getEntityTable() . '} WHERE pid = :pid AND inserted > 0 ORDER BY inserted ASC', array(':pid' => $this->getProfile()->getPid()))->fetchField();
  }

  /**
   * @inheritdoc
   */
  public function batchOperations() {
    $operations = array();

    if ($converters = $this->getConverters()) {
      // Get the total number of objects to map.
      $total = $this->getTotal();
      $maxEntities = $this->getProfile()->getMaxObjectsPerOp();

      if ($total && $maxEntities) {
        for ($i = 0; $i < ceil($total / $maxEntities); $i++) {
          $operations[] = array(
            // Callback function
            array($this, 'process'),
            // Callback arguments
            array(
              $i * $this->getProfile()->getMaxObjectsPerOp(),
              $this->getProfile()->getMaxObjectsPerOp(),
            )
          );
        };
      }
    }

    return $operations;
  }

  /**
   * Map the objects.
   */
  protected function processRun($index = 1, $range = 0) {
    Logger::getLog($this->getType())->setCurrentOperation($index);
    Logger::getLog($this->getType())->setTotalOperations($this->getTotal() + $index);

    if ($entities = $this->getCurrentEntities($range)) {
    	$entityIds = array();

    	foreach ($entities as $id => $data) {
    	  Logger::getLog($this->getType())->incrementCurrentOperation();

    	  // Skip already processed entities (some are related to others).
        if (in_array($id, $entityIds)) {
          continue;
        }

        $entityIds[] = $id;
        $entity = $this->createEntity($data);

        foreach ($this->getConverters() as $converter) {
          if (!$converter->isMappingAllowed()) {
            continue;
          }

          if ($converter->getParentId() > 0) {
            continue;
          }

          //self::$currentMappingType = $converter->getMapping()->getType();

          $property = PropertyWrapper::load($converter, $converter->getUid());
          $entity->setRawValues($property, array(), FALSE);
          $values = $property->getValues($entity);

          if (!$values) {
            continue;
          }

          // @todo use the entity object in matchConditions() instead of the array. So
          // the $data variable will no more be necessary.
          if ($converter->getFilter()->checkConditions($data)) {
          	// @todo get entity UID value. If not possible, log an error and continue with the next object.
            $uid = $values;
            foreach ($this->getEntitiesFromUid($uid, $property->getId()) as $entity_id => $related_entity) {
            	if ($related_entity['id'] === $id) {
              	continue;
              }

              // Ensure the related entity matches the converter criteria.
              if ($converter->getFilter()->checkConditions($related_entity)) {
                $entityIds[] = $entity_id;
                $entity->addRelatedEntity($related_entity);
              }
            }

            self::$log = Logger::getLog($converter->getMapping()->getType());
            self::$log
              ->moveToContentRoot()
              ->addContext(new LogContext(strtolower($converter->getType()), array('id' => $data['id'] . " : " . $converter->getBundle())), 'child');

            $converter->getMapping()->process($entity);

            continue 2;
          }
        }//end foreach

        // @todo log the no_match count using logger.
        //$this->getlog()
        //$this->context['results']['no_match']++;
      }//end foreach

      // Flag all objects as processed, even if no converter matched or
      // an error occured.
      db_update($this->getEntityTable())
        ->fields(array('inserted' => 0))
        ->condition('pid', $this->getProfile()->getPid())
        ->condition('id', $entityIds)
        ->execute();
    }
    // Nothing to do, so we terminate the batch if applicable.
    elseif ($this->isBatch()) {
      $this->context['finished'] = 1;
    }

    Logger::getLog($this->getType())->save();
  }

  /**
   * Get the current list of entities to process.
   */
  protected function getCurrentEntities($maxEntities) {
    $query = $this->selectQuery()
      ->condition('inserted', 0, '>');

    if ($maxEntities > 0) {
    	$query->range(0, $maxEntities);
    }

    return $query->execute()
      ->fetchAllAssoc('id');
  }

  /**
   * Get all entities that share the same UID.
   */
  protected function getEntitiesFromUid($uid, $uid_field) {
    return $this->selectQuery()
      ->condition($uid_field, $uid)
      ->execute()
      ->fetchAllAssoc('id');
  }

  /**
   * Prepare a select query with common configuration.
   *
   * The returned query use the table related to current entity.
   *
   * @return SelectQuery
   *   A new SelectQuery instance.
   */
  protected function selectQuery() {
    return db_select($this->getEntityTable(), 'maps_entity', array('fetch' => \PDO::FETCH_ASSOC))
      ->fields('maps_entity')
      ->condition('pid', $this->getProfile()->getPid());
  }

  /**
   * Create a MaPS Entity from the retrieved data.
   *
   * @param $data
   *   The array containing the raw data.
   *
   * @return EntityInterface
   *   The newly created entity.
   */
  abstract protected function createEntity(array $entity);

  /**
   * Return the log used for the current operation.
   *
   * @return Maps\Log\Log
   *   The current log.
   */
  public static function log() {
    return self::$log;
  }

}
