<?php

/**
 * @file
 * Abstract class that defines a default MaPS System "entity" (object, media, link, ...).
 */

namespace Drupal\maps_import\Mapping\Source\MapsSystem;

use Drupal\maps_import\Cache\Object\Profile as CacheObjectProfile;
use Drupal\maps_import\Mapping\Mapping;
use Drupal\maps_import\Mapping\Mapper\Mapper;
use Drupal\maps_import\Mapping\Source\MapsSystem\Attribute\DefaultAttribute;
use Drupal\maps_import\Mapping\Source\MapsSystem\Attribute\Attribute;
use Drupal\maps_import\Mapping\Source\MapsSystem\Property\Property;
use Drupal\maps_import\Mapping\Source\MapsSystem\Property\Broken as BrokenProperty;
use Drupal\maps_import\Mapping\Source\MapsSystem\Attribute\Broken as BrokenAttribute;
use Drupal\maps_import\Profile\Profile;
use Drupal\maps\Log\Context\Context as LogContext;

abstract class Entity implements EntityInterface {

  /**
   * The profile instance.
   *
   * @var Profile
   */
  protected $profile;

  /**
   * The entity attributes.
   *
   * @var array
   */
  protected $attributes = array();

  /**
   * The entity classes.
   *
   * @var array
   */
  protected $classes = array();

  /**
   * The related entities.
   *
   * @var array
   */
  protected $relatedEntities = array();

  /**
   * The treated values by language
   *
   * @var array
   */
  protected $rawValues = array();

  /**
   * The parent entity.
   *
   * @var Entity
   */
  protected $parentEntity;

  /**
   * @inheritdoc
   */
  public function __construct(array $entity) {
    $this->id = $entity['id'];
    $this->code = !empty($entity['code']) ? $entity['id'] : NULL;

    $this->profile = CacheObjectProfile::getInstance()->loadSingle($entity['pid']);

    foreach (array('classes', 'attributes') as $key) {
      if (isset($entity[$key])) {
        $this->{$key} = unserialize($entity[$key]);
      }
    }//end foreach

    $this->addRelatedEntity($entity);
  }

  /**
   * @inheritdoc
   */
  public function getProfile() {
    return $this->profile;
  }

  /**
   * @inheritdoc
   */
  public function addRelatedEntity(array $entity) {
    if (!isset($this->relatedEntities['id'])) {
      // We add the entity only if it is published.
//      if (!isset($entity['status']) || (int) $this->getProfile()->getPublishedState($entity['status']) == (int) Profile::STATE_PUBLISHED) {
        $this->relatedEntities[$entity['id']] = $this->cleanup($entity);
//      }
    }
  }

  /**
   * @inheritdoc
   */
  public function isPublished() {
    foreach ($this->relatedEntities as $entity) {
      if (!isset($entity['status']) || $this->getProfile()->getPublishedState($entity['status']) === Profile::STATE_PUBLISHED) {
        return TRUE;
      }
    }//end foreach

    return FALSE;
  }

  /**
   * Remove useless entries from the $entity array.
   */
  protected function cleanup(array $entity) {
    foreach (array('classes', 'attributes') as $key) {
      if (isset($entity[$key])) {
        unset($entity[$key]);
      }
    }//end foreach

    return $entity;
  }

  /**
   * @inheritdoc
   */
  public function setRawValues(PropertyWrapperInterface $property, $options = array(), $log = TRUE) {
    if (!array_key_exists($property->getKey(), $this->rawValues)) {
      $this->rawValues[$property->getKey()] = array();
    }

    // Create context if log is set at TRUE.
    if ($log) {
      Mapper::log()->addContext(new LogContext($property->getKey()), 'child');
    }

    $this->rawValues[$property->getKey()] = $property->extractValues($this, $options);

    // Fill context in log.
    if (!$log) {
      return;
    }

    if (empty($this->rawValues[$property->getKey()])) {
      Mapper::log()->moveToParent('properties');
      return;
    }

    foreach ($this->rawValues[$property->getKey()] as $language => $values) {
      if (empty($values)) {
        continue;
      }

      Mapper::log()
        ->addContext(new LogContext('value', array('language' => $language)), 'child');

      if (!is_array($values)) {
        $values = array($values);
      }
      
      foreach ($values as $value) {
        Mapper::log()->addMessage($value);
      }

      Mapper::log()->moveToParent($property->getKey());
    }

    Mapper::log()->moveToParent('properties');
  }

  /**
   * @inheritdoc
   */
  public function hasValues(PropertyWrapperInterface $property, $language_id = 0) {
    if (!$property->isTranslatable()) {
      $language_id = 0;
    }

    return isset($this->rawValues[$property->getKey()])
        && array_key_exists($language_id, $this->rawValues[$property->getKey()])
        && !empty($this->rawValues[$property->getKey()][$language_id]);
  }

  /**
   * @inheritdoc
   */
  public function getAttributes() {
    return $this->attributes;
  }

  /**
   * @inheritdoc
   */
  public function getRelatedEntities() {
    return $this->relatedEntities;
  }

  /**
   * @inheritdoc
   */
  public function getRawValues() {
    return $this->rawValues;
  }

  public function getId() {
    return $this->id;
  }
    
  public function setRelatedEntities(array $relatedEntities) {
  	$this->relatedEntities = $relatedEntities;
  }
}
