<?php

/**
 * @file
 * Manage "Field" translations on Drupal Entity.
 */

namespace Drupal\maps_import\Mapping\Target\Drupal\Translation;

use Drupal\maps_import\Mapping\Source\MapsSystem\EntityInterface as MapsEntityInterface;
use Drupal\maps_import\Mapping\Target\Drupal\EntityInterface;
use Drupal\maps_import\Mapping\Target\Drupal\Field\FieldInterface;
use Drupal\maps_import\Mapping\Source\MapsSystem\PropertyWrapperInterface;
use Drupal\maps_import\Exception\MappingException;
use Drupal\maps_import\Mapping\Mapper\Mapper;
use Drupal\maps\Log\Context\Context as LogContext;

class FieldTranslation extends Translation {

  /**
   * @inheritdoc
   */
  public function __construct(EntityInterface $entity, array $existingEntities = array()) {
    $this->entity = $entity;

    $defaultLanguage = $entity->getProfile()->getDefaultLanguage();

    if (!$defaultLanguage) {
      throw new MappingException('There is no default language.');
    }

    $info = isset($existingEntities[$defaultLanguage]) ? $existingEntities[$defaultLanguage] : array();
    $this->entity->addEntity($defaultLanguage, $info);
  }

  /**
   * @inheritdoc
   */
  public function setValue(FieldInterface $field, PropertyWrapperInterface $property, MapsEntityInterface $mapsEntity, $required = FALSE) {
    $languages = $mapsEntity->getProfile()->getLanguages();

    // There is only one entity in Field translation.
    $wrapper = $this->entity->getEntities();
    $wrapper = reset($wrapper);
    
    $wrapper_language = $wrapper->getPropertyLanguage();

    // Retrieve values for given property in the MaPS entity.
    $values = $property->getValues($mapsEntity);
    if (empty($values)) {
      return;
    }

    // Sanitize values.
    foreach ($values as $languageId => $translatedValue) {
      if ($property->isMultiple()) {
      	if ($field->isMultiple()) {
	        foreach ($translatedValue as $id => $value) {
	          $values[$languageId][$id] = $field->sanitize($value);
	        }
      	}
      	else {
      		// If the property is multiple but not the field, we only use the first value of the property.
	        $values[$languageId] = is_array($translatedValue) ? $field->sanitize(reset($translatedValue)) : $translatedValue;	      		
      	}
      }
      else {
        $value = $field->sanitize($translatedValue);
        $values[$languageId] = $value;
        // Get only the first value if the field is not multiple.
        if (is_array($value) && !$field->isMultiple()) {
        	// Check if it is a text formatted or not.
        	if (!isset($value['value']) && !isset($value['format'])) {
        		$values[$languageId] = reset($value);        		
        	}
        	else {
        		$value['value'] = reset($value['value']);
          	$values[$languageId] = $value;
        	}        	
        }        
      }
    }
  
    // Check if there are values.
    if ((!is_null($values) && !$property->isMultiple())
      || !empty($values) && $property->isMultiple()) {
      	
      Mapper::log()->addContext(new LogContext($field->getId()), 'child');
      	
      foreach ($values as $idLanguage => $value) {
        // Avoid processing null values.
        if (is_null($value) ||
            is_array($value) && isset($value['value']) && is_null($value['value'])) {
          continue;
        }

        try {
          // Retrieve langcode
          $langcode = $mapsEntity->getProfile()->getLangcode($idLanguage);

          if (!is_null($langcode)) {
            $wrapper->language($langcode);
          }

          $value = $property->sanitize($value);
          
          // Set value.
          $wrapper->{$field->getId()}->set($value);

          // Initialize the wrapper language.
          $wrapper->language($wrapper_language->language);
          
          $message = (is_array($values) && isset($values['value'])) ? $values['value'] : $values;
          Mapper::log()
            ->addContext(new LogContext('value'), 'child')
            ->addMessage($message);
        }
        catch (\EntityMetadataWrapperException $e) {
	        Mapper::log()->addContext(new LogContext('error'), 'child');
	          
	        if ($required) {
	          Mapper::log()->addMessageFromException($e);
	        }
	        else {
	          Mapper::log()->addMessage($e->getMessage(), WATCHDOG_WARNING);
	        }
        }
      }
    }

    Mapper::log()->moveToParent('fields');

    return TRUE;
  }

}
