<?php

/**
 * @file
 * Define the MaPS Import Operation base class.
 */

namespace Drupal\maps_import\Operation;

use Drupal\maps\Log\Logger;
use Drupal\maps\Log\Observer\Watchdog as WatchdogObserver;
use Drupal\maps_import\Log\Observer\Mail as MailObserver;
use Drupal\maps_import\Exception\OperationException;
use Drupal\maps_import\Profile\Profile;

/**
 * MaPS Import Operation class.
 */
abstract class Operation implements OperationInterface {

  /**
   * The related maPS Import Profile instance.
   *
   * @var Profile
   */
  protected $profile;

  /**
   * Store the operation context.
   *
   * @var array
   */
  protected $context;

  /**
   * Return the related profile
   *
   * @return Profile
   *   The related profile.
   */
  public function getProfile() {
    return $this->profile;
  }

  /**
   * @inheritdoc
   */
  public function getFullDescription() {
    $output = '<h2>' . $this->getTitle() . '</h2>';
    $description = $this->getDescription();

    if (!empty($description)) {
      $output .= '<p>' . $description . '</p>';
    }

    return $output;
  }

  /**
   * @inheritdoc
   */
  public function isBatch() {
    return isset($this->context['results']['batch_id']);
  }

  /**
   * @inheritdoc
   */
  public function batchOperations() {
    return array(
      array(
        // Callback function
        array($this, 'process'),
        // Callback arguments
        array(),
      ),
    );
  }


  /**
   * @inheritdoc
   */
  public function getTotalOperations(array $args = array()) {
    return 1;
  }

  /**
   * Perform the operation the class is created for.
   *
   * When the operation is run through the batch API, the last argument
   * is the batch context.
   */
  final public function process() {
    $args = func_get_args();

    if ($batch = &batch_get()) {
      $count = func_num_args();
      array_pop($args);

      // func_get_args() does not contain parameters references but a copy
      // of them (except for objects since PHP 5.3).
      $stack = debug_backtrace();
      $context = & $stack[0]['args'][($count - 1)];

      $context['results']['batch_id'] = $batch['id'];
    }
    else {
      $context = array('results' => array('batch_id' => NULL));
    }

    $log = Logger::attachLog($this->getType(), $context['results']['batch_id'])->setIntroductionMessage($this->getTitle())->addObserver(new WatchdogObserver());
    $log->setRelatedToken('maps_import_profile', $this->getProfile());

    $context['results'] += array(
      'messages' => array(),
      'total_op' => 0,
      'class' => get_class($this),
      'operation' => $this->getType(),
      'log' => (string) $log,
    );

    $this->context = &$context;
    $this->context['results']['total_op']++;

    if (($this->isBatch() && (int) variable_get('maps_log_manual', 0) == 1) ||
      (!$this->isBatch() && (int) variable_get('maps_log_drush', 0) == 1 && (int) variable_get('maps_log_drush_global_notifications', 0) == 0)) {
        $log->addObserver(new MailObserver());
    }

    // Let the class do its job.
    call_user_func_array(array($this, 'processRun'), $args);

    $this->logMemoryPeak();
  }

  /**
   * Child classes use this method to perform an operation.
   */
  abstract protected function processRun($index = 1);

  /**
   * @inheritdoc
   */
  public function getlog() {
    if (isset($this->context)) {
      return Logger::getLog($this->getType());
    }

    throw new OperationException('MaPS Import operation %operation: try to get the log object while the context has not been instanciated.', 0, array('%operation' => $this->getType()));
  }

  /**
   * Log the memory usage if necessary.
   */
  protected function logMemoryPeak() {
    $current_peak = NULL;
    $memory_peak = memory_get_peak_usage();

    // @todo find a way to be language neutral. / Lucas: ?
    $formatted = format_size($memory_peak);

    if ($messages = $this->getlog()->moveToContentRoot()->goToContext('memory_peak', TRUE)->getMessages()) {
//      if ((int) $memory_peak > (int) reset($messages)) {
        $this->getlog()->clearAndAddMessage($formatted);
//      }
    }
    else {
      $this->getlog()->addMessage($formatted);
    }
  }

}
