<?php

/**
 * @file
 * Drush integration for the maps_import module.
 */

use Drupal\maps\Log\Logger;
use Drupal\maps_import\Log\Observer\Mail as MailObserver;
use Drupal\maps\Log\Context\Context;
use Drupal\maps_import\Fetcher\Configuration;
use Drupal\maps_import\Fetcher\Objects as ObjectsImport;
use Drupal\maps_import\Mapping\Library\Library;
use Drupal\maps_import\Cache\Object\Profile as CacheObjectProfile;

/**
 * Implements hook_drush_command().
 */
function maps_import_drush_command() {
  $items = array();

  $items['maps-import'] = array(
    'description' => 'Process the given operation.',
    'arguments' => array(
      'pid' => 'The MaPS Import profile id.',
    ),
    'options' => array(
      'op' => 'Specify the operations to process (separated by commas)',
      'list' => 'List the available operations',
    ),
    'examples' => array(
      'maps-import 1' => 'Process all available operations for the profile with the pid 1.',
      'maps-import 1 --op=library_mapping,objects_fetch' => 'Process the "objects_fetch" operation, then the "library_mapping" operation, in the right order.',
      'maps-import 1 --list' => 'List all the available operations for the profile with the pid 1',
    ),
  );

  $items['maps-profile'] = array(
    'description' => 'Manage the MaPS profiles.',
    'options' => array(
      'list' => 'List the availables profiles.',
    ),
  );

  $items['maps-unlock'] = array(
    'description' => 'Manage the MaPS operation lock.',
    'bootstrap' => DRUSH_BOOTSTRAP_DRUSH,
  );

  return $items;
}

/**
 * Command handler. Process the operation.
 *
 * @param int $pid
 *   The MaPS Import profile id.
 */
function drush_maps_import($pid = NULL) {
  if (is_null($pid)) {
    echo "You must specify a profile id as first parameter \n";
    return FALSE;
  }

  if (drush_get_option('list')) {
    drush_maps_import_list($pid);
    return;
  }

  // Check if there is already operations processing.
  if (variable_get('maps_import_lock', 0) == 1) {
    echo "There is already operations under processing.\n";
    return;
  }

  $profile = maps_import_profile_load($pid);
  $operations = maps_import_get_import_operations($profile);

  // Operations defined.
  if ($ops = drush_get_option('op')) {
    $ops = explode(',', $ops);

    // Check if the given operations exists.
    foreach ($ops as $op) {
      $op = trim($op);
      if (!in_array($op, array_keys($operations))) {
        echo "Undefined operation: $op\n";
        return FALSE;
      }
    }

    // Select only choosen operations.
    $to_process = array();
    foreach ($operations as $name => $operation) {
      if (in_array($name, $ops)) {
        $to_process[$name] = $operation;
      }
    }

    $operations = $to_process;
  }

  // Set up lock.
  variable_set('maps_import_lock', 1);

  // Check if we have to log some informations.
  $is_log = (bool) variable_get('maps_log_drush', 0);
  $is_global_log = $is_log && (bool) variable_get('maps_log_drush_global_notifications', 0);

  if ($is_global_log) {
    $summary_log = Logger::attachLog('summary')->addObserver(new MailObserver());
    $summary_log->setRelatedToken('maps_import_profile', $profile);
  }

  // Process chosen operations.
  foreach ($operations as $name => $operation) {
    printf($operation['title'] . "\n");

    $op = new $operation['class']($profile);
    $op->process();

    if ($is_global_log) {
      Logger::getLog('summary')->addContext(new Context($operation['title']), 'child')
        ->addMessage(file_create_url(Logger::getLog($name)->getPath()))
        ->moveToParent();
    }

    maps_import_maps_import_operation_finished($name);
  }

  if ($is_global_log) {
    Logger::getLog('summary')->setCurrentOperation(1);
    Logger::getLog('summary')->save();
  }

  // Remove lock.
  variable_set('maps_import_lock', 0);
}

/**
 * List the available operations.
 *
 * @param int $pid
 *   The MaPS Import profile id.
 */
function drush_maps_import_list($pid) {
  $profile = maps_import_profile_load($pid);
  $operations = maps_import_get_import_operations($profile);

  $mask = $mask = "| %20.20s | %20.20s | %100.100s | \n";
  $row_delimiter = str_pad('', 150, '-') . "\n";

  printf($row_delimiter);
  printf($mask, "Name", "Title", "Description");
  printf($row_delimiter);

  foreach ($operations as $name => $operation) {
    printf($mask, $name, $operation['title'], strip_tags($operation['description']));
  }

  printf($row_delimiter);
}

/**
 * Command handler. Profile management.
 */
function drush_maps_import_maps_profile() {
  // Display available profiles.
  if ($list = drush_get_option('list')) {
    $mask = $mask = "| %5.5s | %20.20s | \n";
    $row_delimiter = str_pad('', 32, '-') . "\n";

    printf($row_delimiter);
    printf($mask, "Pid", "Title");
    printf($row_delimiter);

    foreach (CacheObjectProfile::getInstance()->loadAll() as $pid => $profile) {
      printf($mask, $profile->getPid(), $profile->getTitle());
    }

    printf($row_delimiter);
  }
}

/**
 * Command handler. Manage the MaPS operation lock.
 */
function drush_maps_import_maps_unlock() {
  variable_set('maps_import_lock', 0);
  echo "The lock has been succesfully removed.\n";
}
