<?php

/**
 * @file
 * Provides MaPS integration for MaPS Import module.
 */

use Drupal\maps_import\Profile\Profile;
use Drupal\maps_import\Cache\Object\Profile as CacheObjectProfile;
use Drupal\maps_import\Fetcher\Configuration as ConfigurationFetcher;
use Drupal\maps_import\Fetcher\Objects as ObjectsFetcher;
use Drupal\maps_import\Mapping\Mapper\Object as ObjectMapper;
use Drupal\maps_import\Mapping\Mapper\Media as MediaMapper;
use Drupal\maps_import\Mapping\Library\Library as LibraryMapping;

/**
 * Implements hook_maps_import_entity_presave().
 */
function maps_import_maps_import_entity_presave($entity, $entity_type, $bundle) {
  // Since we may hide some legacy fields if overriden by Title module,
  // the standard behavior of this module will use a non-set value of
  // the legacy field to replace the value of the replacement field.
  // So we need first to restore the legacy field value from the replacement
  // field to avoid such issue.
  if (module_exists('title')) {
    $info = entity_get_info($entity_type);
    $langcode = language_default('language');

    if (!empty($info['field replacement'])) {
      foreach ($info['field replacement'] as $legacy_field => $info) {
        if (title_field_replacement_enabled($entity_type, $bundle, $legacy_field)) {
          $entity->{$legacy_field} = NULL;
          title_field_sync_get($entity_type, $entity, $legacy_field, $info, $langcode);
        }
      }
    }
  }
}

/**
 * Implements hook_maps_admin_overview().
 */
function maps_import_maps_admin_overview() {
  if ($profiles = CacheObjectProfile::getInstance()->loadAll()) {
    $titles = array();
    foreach ($profiles as $profile) {
      $titles[] = $profile->getTitle();
    }

    $content = format_plural(count($profiles), '@count profile: %list.', '@count profiles: %list', array('%list' => implode(', ', $titles)));
  }
  else {
    $content = t('There is no existing profile yet. Add a !link now.', array('!link' => l(t('new profile'), 'admin/maps/profiles/add')));
  }

  return array(t('Web Services profiles') => $content);
}

/**
 * Implements hook_maps_import_profile_actions().
 */
function maps_import_maps_import_profile_actions(Profile $profile) {
  $actions = array(
    'profile-edit' => array(
      'title' => t('edit'),
      'href' => 'admin/maps/profiles/' . $profile->getPid(), 'weight' => -10,
    ),
    'profile-delete' => array(
      'title' => t('delete'),
      'href' => 'admin/maps/profiles/' . $profile->getPid() . '/delete', 'weight' => 10,
    ),
  );

  if ($profile->getConfiguration()) {
    $actions['profile-object-converters'] = array('title' => t('object converters overview'), 'href' => 'admin/maps/profiles/' . $profile->getPid() . '/object');
    $actions['profile-object-converter-add'] = array('title' => t('add object converter'), 'href' => 'admin/maps/profiles/' . $profile->getPid() . '/object/add');
    $actions['profile-media-converters'] = array('title' => t('media converters overview'), 'href' => 'admin/maps/profiles/' . $profile->getPid() . '/media');
    $actions['profile-media-converter-add'] = array('title' => t('add media converter'), 'href' => 'admin/maps/profiles/' . $profile->getPid() . '/media/add');
    $actions['profile-languages'] = array('title' => t('languages mapping'), 'href' => 'admin/maps/profiles/' . $profile->getPid() . '/languages');
    $actions['profile-statuses'] = array('title' => t('statuses mapping'), 'href' => 'admin/maps/profiles/' . $profile->getPid() . '/statuses');
    $actions['profile-media-types'] = array('title' => t('media types mapping'), 'href' => 'admin/maps/profiles/' . $profile->getPid() . '/media_types');
  }

  $actions['profile-import'] = array('title' => t('manual import'), 'href' => 'admin/maps/profiles/' . $profile->getPid() . '/import');

  return $actions;
}

/**
 * Implements hook_maps_import_operations().
 */
function maps_import_maps_import_operations(Profile $profile) {
  $operations = array();

  if ((!empty($GLOBALS['drupal_test_info']['test_run_id']) || !is_null($GLOBALS['drupal_test_info']['test_run_id']))) {
    $instances = array(
      new MapsImportConfigurationFetcherMock($profile),
      new MapsImportObjectsFetcherMock($profile),
    );
  }
  else {
    $instances = array(
      new ConfigurationFetcher($profile),
      new ObjectsFetcher($profile),
      new LibraryMapping($profile),
      new MediaMapper($profile),
      new ObjectMapper($profile),
    );
  }

  drupal_alter('maps_import_operations', $instances, $profile);

  foreach ($instances as $instance) {
    $classname = get_class($instance);

    $operations[$instance->getType()] = array(
      'title' => $instance->getTitle(),
      'description' => $instance->getDescription(),
      'class' => $classname,
    );
  }

  return $operations;
}

/**
 * Implements hook_maps_import_operation_finished().
 */
function maps_import_maps_import_operation_finished($type) {
  switch ($type) {
    case 'configuration_fetch':
      CacheObjectProfile::getInstance()->clearBinCache();
      break;

    case 'object_mapping':
      module_invoke_all('maps_import_mapping_completed');

      // @todo: check !
      if (module_exists('rules')) {
        $args = func_get_args();
        unset($args[0]);
        rules_invoke_event_by_args('maps_import_mapping_completed', $args);
      }
      break;
  }
}
