<?php

/**
 * @file
 * Handle links mapping operation.
 */

namespace Drupal\maps_links\Mapping\Mapper;

use Drupal\maps_import\Fetcher\Fetcher;
use Drupal\maps_import\Mapping\Mapper\Mapper;
use Drupal\maps_links\Mapping\Link as LinkMapping;
use Drupal\maps_links\Mapping\Source\MapsSystem\Link as MapsSystemLink;
use Drupal\maps\Log\Logger;
use Drupal\maps\Log\Context\Context as LogContext;

/**
 * Base class for all mapping operations.
 */
class Link extends Mapper {

  /**
   * @inheritdoc
   */
  public function getTitle() {
    return t('Links mapping');
  }

  /**
   * @inheritdoc
   */
  public function getDescription() {
    return t('The links mapping operation is related to the profile %profile.', array('%profile' => $this->getProfile()->getTitle()));
  }

  /**
   * @inheritdoc
   */
  public function getType() {
    return 'link_mapping';
  }

  /**
   * @inheritdoc
   */
  protected function getConverterClass() {
    return "Drupal\\maps_links\\Converter\\Link";
  }

  /**
   * @inheritdoc
   */
  protected function getEntityTable() {
    return Fetcher::DB_LINKS_TABLE;
  }

  /**
   * @inheritdoc
   */
  protected function createEntity(array $entity) {
    return new MapsSystemLink($entity);
  }

  /**
   * Get the current list of object entities to process
   * and add related medias to objects.
   */
  protected function getCurrentLinks($start, $maxEntities) {
	  $query = db_select($this->getEntityTable(), 'e')
      ->fields('e')
      ->condition('pid', $this->getProfile()->getPid());

    if ($maxEntities > 0) {
      $query->range($start, $maxEntities);    	
    }
      
    return $query->execute()
      ->fetchAllAssoc('id', \PDO::FETCH_ASSOC);
  }
  
  /**
   * @inheritdoc
   */
  public function batchOperations() {
    $operations = array();

    if ($converters = $this->getConverters()) {
      // Get the total number of objects to map.      
      $total = db_select($this->getEntityTable(), 'e')
	      ->fields('e')
	      ->condition('pid', $this->getProfile()->getPid())
	      ->execute()
	      ->rowCount();
	    
      $maxEntities = $this->getProfile()->getMaxObjectsPerOp();
      $count = ceil($total / $maxEntities);
      for ($i = 0; $i < $count; $i++) {
        $operations[] = array(
	        // Callback function
	        array($this, 'process'),
	        // Callback arguments
	        array(
	          $i * $maxEntities,
	          $maxEntities,
	          (bool) ($i == ($count - 1)),
	        )
        );
      }
    }
        
    return $operations;
  }

  /**
   * @inheritdoc
   */
  protected function getConverters() {
//    if (!isset($this->converters)) {
      $this->converters = new \ArrayObject();

      foreach (maps_links_get_converters($this->getProfile()->getPid()) as $converter) {
        $this->converters->append($converter);
      }//end foreach
//    }

    return $this->converters;
  }
  
  public function getTotal() {
    return db_query('SELECT COUNT(*) FROM {' . $this->getEntityTable() . '} WHERE pid = :pid', array(':pid' => $this->getProfile()->getPid()))->fetchField();
  }
  
  /**
   * @inheritdoc
   */
  protected function processRun($start = 0, $maxEntities = 0) {  	    
    Logger::getLog($this->getType())->setCurrentOperation($start);
    Logger::getLog($this->getType())->setTotalOperations($this->getTotal());

    $this->deleteMappedRelations();
  	if ($entities = $this->getCurrentLinks($start, $maxEntities)) {
      foreach ($entities as $id => $data) {        
        Logger::getLog($this->getType())->incrementCurrentOperation();
      	$entity = $this->createEntity($data);
      	
      	foreach ($this->getConverters() as $converter) {
          self::$log = Logger::getLog($converter->getMapping()->getType()); 
      		//self::$currentMappingType = $converter->getMapping()->getType();

      		if ($entity->getLinkType() == $converter->getLinkType()) {            
            self::$log
              ->moveToContentRoot()
              ->addContext(new LogContext(strtolower($converter->getType()), array('id' => $data['source_id'] . " : " . $converter->getBundle())), 'child');
            
            $converter->getMapping()->process($entity);
            
            continue 2;
      		}
      	}      	
      }    	
    }
    
    Logger::getLog($this->getType())->save();
  }
  
  /**
   * Delete all mapped relations created from this profile.
   */
  protected function deleteMappedRelations() {
    // Get all entity ids.
    $ids = db_select(LinkMapping::DB_ENTITIES_TABLE)
      ->fields(LinkMapping::DB_ENTITIES_TABLE, array('id', 'entity_id'))
      ->condition('pid', $this->getProfile()->getPid())
      ->execute()
      ->fetchAllKeyed();

    relation_delete_multiple($ids);         

    db_delete(LinkMapping::DB_ENTITIES_TABLE)
      ->condition('pid', $this->getProfile()->getPid())
      ->execute();
  }
  
}
